@@ -24,7 +24,7 @@
 --------------------------------------------------------------------
 
 {-# LANGUAGE CPP, RankNTypes, MultiParamTypeClasses, FlexibleInstances,
-             GeneralizedNewtypeDeriving, FlexibleContexts #-}
+             GeneralizedNewtypeDeriving, FlexibleContexts, ScopedTypeVariables #-}
 -- The following is needed for generic instances
 {-# LANGUAGE DefaultSignatures, FlexibleContexts, TypeOperators,
              TypeSynonymInstances, FlexibleInstances, OverlappingInstances #-}
@@ -165,6 +165,10 @@ module Test.SmallCheck.Series (
   -- * Basic definitions
   Depth, Series, Serial(..), CoSerial(..),
 
+  -- * Generic implementations
+  genericSeries,
+  genericCoseries,
+
   -- * Convenient wrappers
   Positive(..), NonNegative(..), NonEmpty(..),
 
@@ -174,6 +178,7 @@ module Test.SmallCheck.Series (
   decDepth,
   getDepth,
   generate,
+  limit,
   listSeries,
   list,
   listM,
@@ -204,7 +209,12 @@ class Monad m => Serial m a where
   series   :: Series m a
 
   default series :: (Generic a, GSerial m (Rep a)) => Series m a
-  series = to <$> gSeries
+  series = genericSeries
+
+genericSeries
+  :: (Monad m, Generic a, GSerial m (Rep a))
+  => Series m a
+genericSeries = to <$> gSeries
 
 class Monad m => CoSerial m a where
   -- | A proper 'coseries' implementation should pass the depth unchanged to
@@ -213,8 +223,12 @@ class Monad m => CoSerial m a where
   coseries :: Series m b -> Series m (a->b)
 
   default coseries :: (Generic a, GCoSerial m (Rep a)) => Series m b -> Series m (a->b)
-  coseries rs = (. from) <$> gCoseries rs
+  coseries = genericCoseries
 
+genericCoseries
+  :: (Monad m, Generic a, GCoSerial m (Rep a))
+  => Series m b -> Series m (a->b)
+genericCoseries rs = (. from) <$> gCoseries rs
 -- }}}
 
 ------------------------------
@@ -229,6 +243,18 @@ generate f = do
   d <- getDepth
   msum $ map return $ f d
 
+-- | Limit a 'Series' to its first @n@ elements
+limit :: forall m a . Monad m => Int -> Series m a -> Series m a
+limit n0 (Series s) = Series $ go n0 s
+  where
+    go :: MonadLogic ml => Int -> ml b -> ml b
+    go 0 _ = mzero
+    go n mb1 = do
+      cons :: Maybe (b, ml b) <- msplit mb1
+      case cons of
+        Nothing -> mzero
+        Just (b, mb2) -> return b `mplus` go (n-1) mb2
+
 suchThat :: Series m a -> (a -> Bool) -> Series m a
 suchThat s p = s >>= \x -> if p x then pure x else empty
 
